@@ -269,6 +269,15 @@ print.sendAio <- function(x, ...) {
 
 }
 
+#' @export
+#'
+print.ncurlAio <- function(x, ...) {
+
+  cat("< ncurlAio >\n - $status for response status code\n - $headers for requested response headers\n - $raw for raw message\n - $data for message data\n", file = stdout())
+  invisible(x)
+
+}
+
 #' @export
 #'
 print.unresolvedValue <- function(x, ...) {
@@ -330,3 +339,7 @@ print.errorValue <- function(x, ...) {
 .DollarNames.sendAio <- function(x, pattern = "") grep(pattern, "result",
                                                        value = TRUE, fixed = TRUE)
 
+#' @export
+#'
+.DollarNames.ncurlAio <- function(x, pattern = "") grep(pattern, c("status", "headers", "raw", "data"),
+                                                        value = TRUE, fixed = TRUE)
