@@ -79,7 +79,6 @@ static const R_CallMethodDef CallEntries[] = {
   {"rnng_dialer_create", (DL_FUNC) &rnng_dialer_create, 2},
   {"rnng_dialer_set", (DL_FUNC) &rnng_dialer_set, 4},
   {"rnng_dialer_start", (DL_FUNC) &rnng_dialer_start, 2},
-  {"rnng_hashToChar", (DL_FUNC) &rnng_hashToChar, 1},
   {"rnng_listen", (DL_FUNC) &rnng_listen, 2},
   {"rnng_listener_close", (DL_FUNC) &rnng_listener_close, 1},
   {"rnng_listener_create", (DL_FUNC) &rnng_listener_create, 2},
@@ -96,6 +95,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"rnng_recv", (DL_FUNC) &rnng_recv, 4},
   {"rnng_recv_aio", (DL_FUNC) &rnng_recv_aio, 2},
   {"rnng_scm", (DL_FUNC) &rnng_scm, 0},
+  {"rnng_scm2", (DL_FUNC) &rnng_scm2, 0},
   {"rnng_send", (DL_FUNC) &rnng_send, 4},
   {"rnng_send_aio", (DL_FUNC) &rnng_send_aio, 3},
   {"rnng_serial", (DL_FUNC) &rnng_serial, 1},
