@@ -156,10 +156,10 @@ nanotest(close(surctx) == 0L)
 nanotest(close(surv) == 0L)
 nanotest(close(resp) == 0L)
 
-nanotestwn(ncurl("http://www.r-project.org/"))
+nanotestwn(ncurl("https://www.r-project.org/"))
 nanotestwn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test"))
-nanotestwn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain")))
-put1 <- ncurl("http://httpbin.org/put", async = TRUE, method = "PUT", headers = c(Authorization = "Bearer token"), data = "test")
+nanotestwn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), request = list("Date", "Server")))
+put1 <- ncurl("http://httpbin.org/put", async = TRUE, method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", request = c("date", "server"))
 nanotestn(put1$raw)
 put2 <- ncurl("http://httpbin.org/put", async = TRUE, method = "PUT", headers = list(Authorization = "Bearer token"))
 nanotestn(call_aio(put2)$data)
@@ -174,7 +174,12 @@ nanotestwn(messenger("invalidURL"))
 nanotestn(tryCatch(.mirai_scm(), error = function(e) 0L))
 nanotest(is.double(mclock()))
 nanotest(is.null(msleep(1)))
+nanotest(is.null(msleep("a")))
 nanotest(is.double(random()))
+nanotest(length(sha256("test")) == 32L)
+nanotest(is.raw(sha256(c(1.1, 2.2), as.raw(1L))))
+nanotest(inherits(sha256(data.frame(), "KEY"), "nanoHash"))
+nanotest(length(as.character(sha256(as.raw(0L), key = NULL))) == 1L)
 
 nanotest(inherits(bus <- socket(protocol = "bus"), "nanoSocket"))
 nanotest(inherits(push <- socket(protocol = "push"), "nanoSocket"))
